#!/usr/bin/perl -w
# VIM: set ts=4 set sw=4
#
# whois.pl - beautify xchat's /WHOIS and /WHOWAS printings
#
package IRC::XChat::Whois;

# set the foreground color of the output
my $color = "\cC0";

my $script_name    = "whois.pl";
my $script_version = '0.1.2';

my $hacked = "\cB-=[\cBhacked 07/2002 by \cB\cC10Z\cC3irias\cO\cB]=-\cB";

IRC::register($script_name, $script_version, "", "");
IRC::print("\cC0\cB$script_name\cB version\cC3 $script_version\cO "
           . "by \cB\cC4V\cC7etinari\cO $hacked loading...\n");

my $pkg = __PACKAGE__;
my (%request, $whowas_query)  = ((),'');

IRC::add_command_handler("WHOIS", "${pkg}::handle_whois");
IRC::add_command_handler("WHOWAS","${pkg}::handle_whowas");

# RPL_AWAY:
IRC::add_message_handler("301", "${pkg}::handle_reply");
# RPL_WHOISUSER:
IRC::add_message_handler("311", "${pkg}::handle_reply");
# RPL_WHOISSERVER
IRC::add_message_handler("312", "${pkg}::handle_reply"); 
# RPL_WHOISOPERATOR:
IRC::add_message_handler("313", "${pkg}::handle_reply");
# RPL_WHOWASUSER:
IRC::add_message_handler("314", "${pkg}::handle_reply"); 
# RPL_ENDOFWHOIS:
IRC::add_message_handler("318", "${pkg}::handle_reply");
# RPL_WHOISCONN:
IRC::add_message_handler("378", "${pkg}::handle_reply");
# RPL_WHOISREG:
IRC::add_message_handler("307", "${pkg}::handle_reply");
# RPL_WHOISHELP:
IRC::add_message_handler("310", "${pkg}::handle_reply");
# RPL_WHOISSSL:
IRC::add_message_handler("320", "${pkg}::handle_reply");
# RPL_WHOISMETA:
IRC::add_message_handler("335", "${pkg}::handle_reply");
# RPL_WHOISIDLE:
IRC::add_message_handler("317", "${pkg}::handle_reply");
# RPL_WHOISCHANNELS:
IRC::add_message_handler("319", "${pkg}::handle_reply");
# RPL_ENDOFWHOWAS:
IRC::add_message_handler("369", "${pkg}::handle_reply"); 
# ERR_NOSUCHNICK:
IRC::add_message_handler("401", "${pkg}::handle_reply");
# ERR_WASNOSUCHNICK:
IRC::add_message_handler("406", "${pkg}::handle_reply");

### Utilities:
sub trim {
	local($_) = shift;
	s/^\s*//;
	s/\s*$//;
	return $_;
}

# lower(NICK);
#   -> return an IRC conforming lower cased nick name
sub lower ($) {
    local($_) = shift;
    tr/A-Z[]\\~/a-z{}|^/;
    return $_;
}

sub handle_whois {
	my $list = trim(shift);
	my ($target,$nicks) = split /\s+/, $list, 2;
	if (!defined $nicks) {
		$list = $target;
		$target = "";
	} else {
		$target .= ' ';
		$list   = $nicks;
	}
	if ($list =~ /\s/) {
		$list = s/\s+.*$//;
	}
    my ($chan,$serv) = (IRC::get_info(2),IRC::get_info(3));
    if ($chan eq "") {
        $chan = $serv;
    }
	foreach my $nick (split ',', $list) {
		my $lo_nick = lower($nick);
		$request{$lo_nick}->{'command'} = 'WHOIS';
        $request{$lo_nick}->{'w_channel'} = $chan;
        $requets{$lo_nick}->{'w_server'}  = $serv;
	}
	if ($target =~ /^$/ && $list !~ /,/) {
		$target = "$list ";
	}
	IRC::send_raw("WHOIS $target$list\r\n");
	return 1;
}


sub handle_whowas {
	my $list = trim(shift);
	my ($nicks,$max,$target) = split /\s+/, $list;
	my @list = split /,/, $nicks;
    my ($chan,$serv) = (IRC::get_info(2),IRC::get_info(3));
    if ($chan eq "") {
        $chan = $serv;
    }
	foreach my $nick (@list) {
		my $lo_nick = lower($nick);
		$request{$lo_nick}->{'command'} = 'WHOWAS';
        $request{$lo_nick}->{'w_channel'} = $chan;
        $request{$lo_nick}->{'w_server'}  = $serv;
	}
	if (defined $max) {
		$max = " $max";
	} else {
		$max = "";
	}
	if (defined $target) {
		$target = " $target";
	} else {
		$target = "";
	}
	$whowas_query = $nicks;
	IRC::send_raw("WHOWAS $nicks$max$target\r\n");
	return 1;
}

sub handle_reply {
	my $line = shift;
	$line =~ /:\S+\s+(\S+)\s+\S+\s+(.*?)\s+:(.*)/;
	my ($num,$nick_info,$text) = ($1,$2,$3);
	my $subname = "${pkg}::message_$num";
	return &$subname($nick_info, $text);
}

# RPL_WHOISCONN:
sub message_378 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	$text =~ /.* (\*\@)?(.+)/;
	my $host = $2;
	$request{$lo_nick}->{conn} = $host;
	return 1;
}

# RPL_WHOISREG:
sub message_307 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	$request{$lo_nick}->{reg} = $text;
	return 1;
}

# RPL_WHOISHELP:
sub message_310 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	$request{$lo_nick}->{help} = $text;
	return 1;
}

# RPL_WHOISSSL:
sub message_320 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	$request{$lo_nick}->{ssl} = $text;
	return 1;
}

# RPL_WHOISMETA:
sub message_335 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	$request{$lo_nick}->{meta} = $text;
	return 1;
}

# RPL_AWAY:
sub message_301 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHO(IS|WAS)$/;
	$request{$lo_nick}->{away} = $text;
	return 1;
}

# WHOISUSER
sub message_311 {
	my ($nick_info,$irc_name) = @_;
	my ($nick,$user,$host,$text) = split /\s+/, $nick_info;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	$request{$lo_nick}->{name} = $irc_name;
	$request{$lo_nick}->{host} = "$user\@$host";
	$request{$lo_nick}->{nick} = $nick;
	return 1;
}

# WHOISSERVER
sub message_312 {
	my ($server,$info) = @_;
	my $nick;
	$server =~ /(\S+)\s+(\S+)/;
	($nick,$server) = ($1,$2);
	my $lo_nick = lower($nick);
	my $cmd = $request{$lo_nick}->{'command'} || "";
	if ($cmd =~ /^WHOIS$/) {
		$request{$lo_nick}->{server} = "$server ($info)";
		return 1;
	} 
	elsif ($cmd =~ /^WHOWAS$/) {
		push @{$request{$lo_nick}->{server}}, $server;
		push @{$request{$lo_nick}->{date}},   $info;
		return 1;
	}
	return undef;
}

# WHOISOPERATOR
sub message_313 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	$request{$lo_nick}->{oper} = $text;
	return 1;
}

# WHOWASUSER
sub message_314 {
	my ($nick_info,$irc_name) = @_;
	my ($nick,$user,$host,$text) = split /\s+/, $nick_info;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOWAS$/;
	push @{$request{$lo_nick}->{name}}, $irc_name;
	push @{$request{$lo_nick}->{host}}, "$user\@$host";
	push @{$request{$lo_nick}->{nick}}, $nick;
	return 1;
}

# WHOISIDLE
# quakenet,... (I have also seen this on an ircnet server...)
# >> :irc.de.quakenet.eu.org 317 OhWeia OhWeia 556 984225388 :seconds idle, signon time
#                                       ^------------------^
# ircnet, ... 
# >> :freenet.de 317 Vetinari Vetinari 5 :seconds idle
#                             ^--------^
sub message_317 {
	my $nick = shift;
	my ($time,$signon);
	if ($nick =~ /(\S+)\s+(\d+)\s+(\d+)/) {
		($nick,$time,$signon) = ($1,$2,$3);
		my $lo_nick = lower($nick);
		return undef
			unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
		$request{$lo_nick}->{idle} = $time;
		$request{$lo_nick}->{signon} = $signon;
		return 1;
	}
	$nick =~ /(.*)\s+(\d+)/;
	($nick,$time) = ($1,$2);
	$lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	$request{$lo_nick}->{idle} = $time;
	return 1;
}

# ENDOFWHOIS
sub message_318 {
	my $nick = shift;
	$lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	if ($request{$lo_nick}->{NOSUCHNICK}) {
		delete $request{$lo_nick};
		return 1;
	}
	$nick = $request{$lo_nick}->{nick};
    my $serv = $request{$lo_nick}->{w_server};
    my $chan = $request{$lo_nick}->{w_channel};
    # IRC::print("Server: $serv Channel: $chan Nick: $nick\n");
	IRC::print_with_channel("\cB$color\015\cC9-----\cB-\cB-\cB-[ "
			   . "\cC4whois $color\cB$nick\cB \cC9]--\cC3\cB"
			   . "---\cB-\cB-\cB----\cC14\cB----\cB-----"
			   . "\cB----\cB----\n",
               $chan, $serv
			);
	IRC::print_with_channel("\cB\cC9|$color u\cBserhost\cB:\cB "
			   . $request{$lo_nick}->{host} 
			   . "\n",
               $chan, $serv
			);
	if ($request{$lo_nick}->{conn}) {
		IRC::print_with_channel("\cB\cC9|$color r\cBealhost\cB:\cB "
				. $request{$lo_nick}->{conn}
				. "\n",
				$chan, $serv);
		}
	IRC::print_with_channel("\cC3|$color \cB i\cBrcname\cB:\cB "
			   . $request{$lo_nick}->{name}
			   . "\n", $chan, $serv)
		 if $request{$lo_nick}->{name};
	if ($request{$lo_nick}->{reg}) {
		IRC::print_with_channel("\cC3|$color     \cBn\cBick\cB:\cB "
				. "$nick "
				. $request{$lo_nick}->{reg}
				. "\n",
				$chan, $serv);
		}

	if (scalar(@{ $request{$lo_nick}->{channel} })) {
		foreach my $chans (@{ $request{$lo_nick}->{channel} }) {
			IRC::print_with_channel("\cC3|$color \cB c\cBhannel\cB:\cB $chans\n",
                                   $chan, $serv);
		}
	}
	if ($request{$lo_nick}->{signon}) {
		
		IRC::print_with_channel("\cC3|$color \cB    i\cBdle\cB:\cB "
				   . duration($request{$lo_nick}->{idle}) .", "
				   . "\cBS\cBign\cBO\cBn\cB:\cB "
				   . scalar(gmtime($request{$lo_nick}->{signon})). "\n",
                   $chan, $serv); 

	} else {
		if ($request{$lo_nick}->{idle}) {
			$idle = $request{$lo_nick}->{idle};
			IRC::print_with_channel("\cC3|$color \cB    i\cBdle\cB:\cB ".
                                   duration($idle)."\n", $chan, $serv);
		}
	}
	IRC::print_with_channel("\cC14|$color \cB    a\cBway\cB:\cB "
			   . $request{$lo_nick}->{away} . "\n", $chan, $serv) 
		if $request{$lo_nick}->{away};
	IRC::print_with_channel("\cC14|$color \cBo\cBperator\cB:\cB $nick " 
			   . $request{$lo_nick}->{oper} . "\n", $chan, $serv) 
		if $request{$lo_nick}->{oper};
	IRC::print_with_channel("\cC14|$color   \cBh\cBelper\cB:\cB $nick " 
			   . $request{$lo_nick}->{help} . "\n", $chan, $serv) 
		if $request{$lo_nick}->{help};
	IRC::print_with_channel("\cC14|$color      \cBs\cBsl\cB:\cB $nick " 
			   . $request{$lo_nick}->{ssl} . "\n", $chan, $serv) 
		if $request{$lo_nick}->{ssl};
	IRC::print_with_channel("\cC14|$color      \cBb\cBot\cB:\cB $nick " 
			   . $request{$lo_nick}->{meta} . "\n", $chan, $serv) 
		if $request{$lo_nick}->{meta};
	IRC::print_with_channel("\cC14|$color \cB  s\cBerver\cB:\cB "
			   . $request{$lo_nick}->{server} . "\n", $chan, $serv);

	delete $request{$lo_nick};
	return 1;
}

sub duration {
	my $time = shift;
	my $hrs  = int($time/3600);
	$time    = $time % 3600;
	my $min  = int($time/60);
	$time    = $time % 60;
    return sprintf("$hrs:%02d:%02d", $min, $time); 
}
	
# WHOISCHANNELS
sub message_319 {
	my ($nick,$channels) = @_;
	my $lo_nick = lower($nick);
	return undef
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
	push @{$request{$lo_nick}->{channel}}, $channels;
	return 1;
}

# ENDOFWHOWAS
sub message_369 {
	my $names = shift;
	return undef
		unless $names eq $whowas_query;
	$whowas_query = "";
	my @list = split /,/, $names;
	foreach my $nick (@list) {
		my $lo_nick = lower($nick);
		if ($request{$lo_nick}->{WASNOSUCHNICK}) {
			delete $request{$lo_nick};
			# IRC::print("-\cC10-\cC11-$color whowas: \cB$nick\cB: "
			#		   . "There was no such nickname\n");
		}
        my $serv = $request{$lo_nick}->{w_server};
        my $chan = $request{$lo_nick}->{w_channel};
		my $i = 0;
		while (scalar(@{$request{$lo_nick}->{server}}) > $i) {
			$nick = $request{$lo_nick}->{nick}[$i];
			IRC::print_with_channel("\cB$color\015\cC9-----------\cB----["
                       ."\cC4whowas \cB$color$nick\cC9]----\cC3\cB"
					   . "------\cB-----\cC14\cB------\cB-------\n",
                       $chan, $serv);
			IRC::print_with_channel("\cB\cC9|$color h\cBostname\cB:\cB "
					   . "$request{$lo_nick}->{host}->[$i]\n",
                       $chan, $serv);
			IRC::print_with_channel("\cC3|$color \cB i\cBrcname\cB:\cB "
					   . "$request{$lo_nick}->{name}->[$i]\n",
                       $chan, $serv)
		 		if $request{$lo_nick}->{name}[$i];
			IRC::print_with_channel("\cC14|$color \cB  s\cBerver\cB:\cB "
					   . "$request{$lo_nick}->{server}->[$i] ("
					   . "$request{$lo_nick}->{date}->[$i])\n",
                       $chan, $serv);
			++$i;
		}
		delete $request{$lo_nick};
	}
	return 1;
}
	
# NOSUCHNICK
sub message_401 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef 
		unless $request{$lo_nick}->{'command'} =~ /^WHOIS$/;
    my $chan = $request{$lo_nick}->{w_channel};
    my $serv = $request{$lo_nick}->{w_server};
	IRC::print_with_channel("-\cC10-\cC11-$color whois: \cB$nick\cB: $text\n",
                            $chan, $serv);
	$request{$lo_nick}->{NOSUCHNICK} = 1;
	return 1;
}

# WASNOSUCHNICK
sub message_406 {
	my ($nick,$text) = @_;
	my $lo_nick = lower($nick);
	return undef 
		unless $request{$lo_nick}->{'command'} =~ /^WHOWAS$/;
    my $chan = $request{$lo_nick}->{w_channel};
    my $serv = $request{$lo_nick}->{w_server};
	IRC::print_with_channel("-\cC10-\cC11-$color whowas: \02$nick\02: $text\n",
                            $chan, $serv);
	$request{$lo_nick}->{WASNOSUCHNICK} = 1;
	return 1;
}

1;
# end 
